#!/bin/bash
#

# build_numpy.sh 

# The build is qualified because it depends upon LAPACK, which is qualified.
usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e10|e12|e14>:<p2711|p2713c|p2713d> <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

get_pqual()
{
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
	case $qual in
            e9|e10|e12|e14)
        	cqual=$qual
        	;;
            p2710|p2711|p2713c|p2713d)
        	pqual=$qual
        	;;
            *)
        	ssi_die "Unrecognized qualifier $qual"
	esac
    done
    if [[ -z "$cqual" ]]; then
	ssi_die "Base qualifier is mandatory"
    fi
    if [[ -z "$pqual" ]]; then
	ssi_die "A python qualifier is mandatory (e.g. p2711)."
    fi
}

set_compiler_flags()
{
  case ${extraqual} in
    opt)
      cflags="-O3 -g -DNDEBUG"
      ;;
    debug)
      cflags="-g -O0"
      ;;
    prof)
      cflags="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
      ;;
  esac
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=numpy
origpkgver=v1_11_2
pkgver=${origpkgver}a
ssibuildshims_version=v0_15_09
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims

source define_basics

set_compiler_flags
get_pqual

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

set -x

# work in a build directory, which will not be part of any distribution
python_shortdotversion=${PYTHON_VERSION#v}
python_shortdotversion=${python_shortdotversion//_/.}
python_shortdotversion=${python_shortdotversion%.*}

mkdir -p ${pkgdir}/lib/python${python_shortdotversion}/site-packages || ssi_die "failed to create ${pkgdir}/lib/python${python_shortdotversion}/site-packages"
mkdir -p ${blddir} || ssi_die "failed to create ${blddir}"

cd ${blddir} || ssi_die "Unable to cd into ${blddir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || ssi_die "Failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || ssi_die "Failed to cd into ${package}-${pkgdotver}"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

FC=false \
CC=gcc \
CXX=false \
CFLAGS="${cflags}" \
python setup.py build -j ${ncore} install --prefix ${pkgdir} || ssi_die "python setup.py install failed"

# run some tests
nosetests -w ${pkgdir}/lib/python${python_shortdotversion}/site-packages/numpy*/numpy/tests || ssi_die "nosetests failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

#replace hardwired python path 
sed -i'' -e '1s/^.*$/#!\/usr\/bin\/env python /' ${pkgdir}/bin/f2py || ssi_die "Failed to fix shebang for f2py"

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "numpy is installed at ${NUMPY_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
